@@ -28,12 +28,12 @@
 #' the function returns a list containing
 #' \describe{
 #' \item{$geno$pop.geno}{the genotype data.}
-#' \item{$geno$inrows}{'1': one-row genotype represents an individual; '2': two-row genotype represents an individual.}
+#' \item{$geno$inrows}{"1": one-row genotype represents an individual; "2": two-row genotype represents an individual.}
 #' \item{$geno$pop.marker}{the number of markers.}
 #' \item{$geno$pop.ind}{the number of individuals in the base population.}
 #' \item{$geno$prob}{the genotype code probability.}
 #' \item{$geno$rate.mut}{the mutation rate of the genotype data.}
-#' \item{$geno$cld}{whether to generate a complete LD genotype data when 'inrows == 2'.}
+#' \item{$geno$cld}{whether to generate a complete LD genotype data when "inrows == 2".}
 #' }
 #' 
 #' @export
@@ -214,10 +214,10 @@ genotype <- function(SP = NULL, ncpus = 0, verbose = TRUE) {
 #' \item{$map$pop.marker}{the number of markers.}
 #' \item{$map$num.chr}{the number of chromosomes.}
 #' \item{$map$len.chr}{the length of chromosomes.}
-#' \item{$map$qtn.model}{the genetic model of QTN such as 'A + D'.}
+#' \item{$map$qtn.model}{the genetic model of QTN such as "A + D".}
 #' \item{$map$qtn.index}{the QTN index for each trait.}
 #' \item{$map$qtn.num}{the QTN number for (each group in) each trait.}
-#' \item{$map$qtn.dist}{the QTN distribution containing 'norm', 'geom', 'gamma' or 'beta'.}
+#' \item{$map$qtn.dist}{the QTN distribution containing "norm", "geom", "gamma" or "beta".}
 #' \item{$map$qtn.var}{the variances for normal distribution.}
 #' \item{$map$qtn.prob}{the probability of success for geometric distribution.}
 #' \item{$map$qtn.shape}{the shape parameter for gamma distribution.}
@@ -253,6 +253,7 @@ annotation <- function(SP, verbose = TRUE) {
   num.chr <- SP$map$num.chr
   len.chr <- SP$map$len.chr
   pop.geno <- SP$geno$pop.geno
+  inrows <- SP$geno$inrows
   qtn.model <- SP$map$qtn.model
   qtn.index <- SP$map$qtn.index
   qtn.num <- SP$map$qtn.num
@@ -339,10 +340,17 @@ annotation <- function(SP, verbose = TRUE) {
     if (is.null(pop.geno)) {
       stop("MAF calculation need genotype data!")
     }
-    if (nrow(pop.geno) != nrow(pop.map)) {
+    if (ncol(pop.geno) != nrow(pop.map)) {
       stop("Marker number should be same in both 'pop.map' and 'pop.geno'!")
     }
-    MAF <- rowSums(pop.geno[]) / ncol(pop.geno)
+    if (inrows == 1) {
+      MAF <- colSums(pop.geno[]) / (2*nrow(pop.geno))
+    } else if (inrows == 2) {
+      MAF <- colSums(pop.geno[]) / nrow(pop.geno)
+    } else {
+      stop("'inrows' should only be 1 or 2!")
+    }
+    
     MAF <- pmin(MAF, 1 - MAF)
     pop.map$QTNProb[MAF < maf] <- 0
     pop.map$MAF <- MAF
@@ -422,7 +430,7 @@ annotation <- function(SP, verbose = TRUE) {
 #' @author Dong Yin
 #'
 #' @param qtn.num integer: the QTN number of single trait; vector: the multiple group QTN number of single trait; matrix: the QTN number of multiple traits.
-#' @param qtn.dist the QTN distribution containing 'norm', 'geom', 'gamma' or 'beta'.
+#' @param qtn.dist the QTN distribution containing "norm", "geom", "gamma" or "beta".
 #' @param qtn.var the standard deviations for normal distribution.
 #' @param qtn.prob the probability of success for geometric distribution.
 #' @param qtn.shape the shape parameter for gamma distribution.
