@@ -1,35 +1,30 @@
-#include <RcppArmadillo.h>
-#include "simer_omp.h"
-#include <bigmemory/BigMatrix.h>
-#include <bigmemory/MatrixAccessor.hpp>
-#include <progress.hpp>
-
-// [[Rcpp::plugins(cpp11)]]
-// [[Rcpp::depends(RcppArmadillo)]]
-// [[Rcpp::depends(bigmemory, BH)]]
-// [[Rcpp::depends(RcppProgress)]]
+#include "simer.h"
 
 using namespace std;
 using namespace Rcpp;
 using namespace arma;
 
+// [[Rcpp::depends(RcppArmadillo)]]
+// [[Rcpp::depends(bigmemory, BH)]]
+// [[Rcpp::depends(RcppProgress)]]
+
 template<typename T>
 NumericVector FilterMind(XPtr<BigMatrix> pMat, double NA_C, int threads=0) {
   omp_setup(threads);
   
   MatrixAccessor<T> bigm = MatrixAccessor<T>(*pMat);
   
-  size_t i, j, m = pMat->nrow(), n = pMat->ncol();
-  NumericVector colNumNA(n, 0);
+  size_t i, j, n = pMat->nrow(), m = pMat->ncol();
+  NumericVector indNumNA(n, 0);
   
   #pragma omp parallel for schedule(dynamic) private(i, j)
-  for (j = 0; j < n; j++) {
-    for (i = 0; i < m; i++) {
-      if (bigm[j][i] == NA_C) { colNumNA[j] += 1;  }
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      if (bigm[j][i] == NA_C) { indNumNA[i] += 1;  }
     }
   }
   
-  return colNumNA;
+  return indNumNA;
 }
 
 NumericVector FilterMind(const SEXP pBigMat, double NA_C, int threads=0) {
@@ -50,58 +45,58 @@ NumericVector FilterMind(const SEXP pBigMat, double NA_C, int threads=0) {
 }
 
 template<typename T>
-NumericVector FilterGeno(XPtr<BigMatrix> pMat, double NA_C, IntegerVector rowIdx, IntegerVector colIdx, int threads=0) {
+NumericVector FilterGeno(XPtr<BigMatrix> pMat, double NA_C, IntegerVector &indIdx, IntegerVector &mrkIdx, int threads=0) {
   omp_setup(threads);
   
   MatrixAccessor<T> bigm = MatrixAccessor<T>(*pMat);
-  size_t i, j;
-  NumericVector rowNumNA(rowIdx.size(), 0);
+  size_t i, j, n = indIdx.size(), m = mrkIdx.size();
+  NumericVector mrkNumNA(m, 0);
   
   #pragma omp parallel for schedule(dynamic) private(i, j)
-  for (j = 0; j < colIdx.size(); j++) {
-    for (i = 0; i < rowIdx.size(); i++) {
-      if (bigm[colIdx[j]][rowIdx[i]] == NA_C) { rowNumNA[i] += 1;  }
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      if (bigm[mrkIdx[j]][indIdx[i]] == NA_C) { mrkNumNA[j] += 1;  }
     }
   }
   
-  return rowNumNA;
+  return mrkNumNA;
 }
 
-NumericVector FilterGeno(const SEXP pBigMat, double NA_C, IntegerVector rowIdx, IntegerVector colIdx, int threads=0) {
+NumericVector FilterGeno(const SEXP pBigMat, double NA_C, IntegerVector &indIdx, IntegerVector &mrkIdx, int threads=0) {
   XPtr<BigMatrix> xpMat(pBigMat);
   
   switch(xpMat->matrix_type()) {
   case 1:
-    return FilterGeno<char>(xpMat, NA_CHAR, rowIdx, colIdx, threads);
+    return FilterGeno<char>(xpMat, NA_CHAR, indIdx, mrkIdx, threads);
   case 2:
-    return FilterGeno<short>(xpMat, NA_SHORT, rowIdx, colIdx, threads);
+    return FilterGeno<short>(xpMat, NA_SHORT, indIdx, mrkIdx, threads);
   case 4:
-    return FilterGeno<int>(xpMat, NA_INTEGER, rowIdx, colIdx, threads);
+    return FilterGeno<int>(xpMat, NA_INTEGER, indIdx, mrkIdx, threads);
   case 8:
-    return FilterGeno<double>(xpMat, NA_REAL, rowIdx, colIdx, threads);
+    return FilterGeno<double>(xpMat, NA_REAL, indIdx, mrkIdx, threads);
   default:
     throw Rcpp::exception("unknown type detected for big.matrix object!");
   }
 }
 
 template<typename T>
-arma::mat CalGenoFreq(XPtr<BigMatrix> pMat, double NA_C, IntegerVector rowIdx, IntegerVector colIdx, int threads=0) {
+arma::mat CalGenoFreq(XPtr<BigMatrix> pMat, double NA_C, IntegerVector &indIdx, IntegerVector &mrkIdx, int threads=0) {
   omp_setup(threads);
   
   MatrixAccessor<T> bigm = MatrixAccessor<T>(*pMat);
   
-  size_t i, j;
-  arma::mat genoFreq(rowIdx.size(), 3, fill::zeros);
+  size_t i, j, n = indIdx.size(), m = mrkIdx.size();
+  arma::mat genoFreq(m, 3, fill::zeros);
 
   #pragma omp parallel for schedule(dynamic) private(i, j)
-  for (i = 0; i < rowIdx.size(); i++) {
-    for (j = 0; j < colIdx.size(); j++) {
-      if (bigm[colIdx[j]][rowIdx[i]] == 0) {
-        genoFreq(i, 0) = genoFreq(i, 0) + 1; 
-      } else if (bigm[colIdx[j]][rowIdx[i]] == 1) {
-        genoFreq(i, 1) = genoFreq(i, 1) + 1;
-      } else if (bigm[colIdx[j]][rowIdx[i]] == 2) {
-        genoFreq(i, 2) = genoFreq(i, 2) + 1;
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      if (bigm[mrkIdx[j]][indIdx[i]] == 0) {
+        genoFreq(j, 0) = genoFreq(j, 0) + 1; 
+      } else if (bigm[mrkIdx[j]][indIdx[i]] == 1) {
+        genoFreq(j, 1) = genoFreq(j, 1) + 1;
+      } else if (bigm[mrkIdx[j]][indIdx[i]] == 2) {
+        genoFreq(j, 2) = genoFreq(j, 2) + 1;
       }
     }
   }
@@ -109,37 +104,37 @@ arma::mat CalGenoFreq(XPtr<BigMatrix> pMat, double NA_C, IntegerVector rowIdx, I
   return genoFreq;
 }
 
-arma::mat CalGenoFreq(const SEXP pBigMat, IntegerVector rowIdx, IntegerVector colIdx, int threads=0) {
+arma::mat CalGenoFreq(const SEXP pBigMat, IntegerVector &indIdx, IntegerVector &mrkIdx, int threads=0) {
   XPtr<BigMatrix> xpMat(pBigMat);
   
   switch(xpMat->matrix_type()) {
   case 1:
-    return CalGenoFreq<char>(xpMat, NA_CHAR, rowIdx, colIdx, threads);
+    return CalGenoFreq<char>(xpMat, NA_CHAR, indIdx, mrkIdx, threads);
   case 2:
-    return CalGenoFreq<short>(xpMat, NA_SHORT, rowIdx, colIdx, threads);
+    return CalGenoFreq<short>(xpMat, NA_SHORT, indIdx, mrkIdx, threads);
   case 4:
-    return CalGenoFreq<int>(xpMat, NA_INTEGER, rowIdx, colIdx, threads);
+    return CalGenoFreq<int>(xpMat, NA_INTEGER, indIdx, mrkIdx, threads);
   case 8:
-    return CalGenoFreq<double>(xpMat, NA_REAL, rowIdx, colIdx, threads);
+    return CalGenoFreq<double>(xpMat, NA_REAL, indIdx, mrkIdx, threads);
   default:
     throw Rcpp::exception("unknown type detected for big.matrix object!");
   }
 }
 
-NumericVector FilterMAF(arma::mat genoFreq, int threads=0) {
+NumericVector FilterMAF(arma::mat &genoFreq, int threads=0) {
   omp_setup(threads);
   
   IntegerVector freq0 = wrap(genoFreq.col(0));
   IntegerVector freq1 = wrap(genoFreq.col(1));
   IntegerVector freq2 = wrap(genoFreq.col(2));
 
-  size_t i;
-  NumericVector MAF(genoFreq.n_rows); MAF.fill(0);
+  size_t i, m = genoFreq.n_rows;
+  NumericVector MAF(m); MAF.fill(0);
   
   #pragma omp parallel for schedule(dynamic) private(i)
-  for (i = 0; i < genoFreq.n_rows; i++) {
+  for (i = 0; i < m; i++) {
     MAF[i] = (freq0[i] + freq1[i] * 0.5) / 
-      (freq0[i]+ freq1[i] + freq2[i]);
+      (freq0[i] + freq1[i] + freq2[i]);
     MAF[i] = MAF[i] <= 0.5 ? MAF[i] : (1 - MAF[i]);
   }
   
@@ -220,35 +215,38 @@ double SNPHWE(int obs_hets, int obs_hom1, int obs_hom2) {
   return p_hwe;
 }
 
-NumericVector FilterHWE(arma::mat genoFreq, int threads=0) {
+NumericVector FilterHWE(arma::mat &genoFreq, int threads=0) {
   omp_setup(threads);
   
-  size_t i;
+  size_t i, m = genoFreq.n_rows;
   IntegerVector freq0 = wrap(genoFreq.col(0));
   IntegerVector freq1 = wrap(genoFreq.col(1));
   IntegerVector freq2 = wrap(genoFreq.col(2));
-  NumericVector PVAL(genoFreq.n_rows); PVAL.fill(0);
+  NumericVector PVAL(m); PVAL.fill(0);
   
   #pragma omp parallel for schedule(dynamic) private(i)
-  for (i = 0; i < genoFreq.n_rows; i++) {
+  for (i = 0; i < m; i++) {
     PVAL[i] = SNPHWE(freq1[i], freq0[i], freq2[i]);
   }
   
   return PVAL;
 }
 
-template<typename T>
-List GenoFilter(XPtr<BigMatrix> pMat, double NA_C, Nullable<IntegerVector> keepInds=R_NilValue, Nullable<double> filterGeno=R_NilValue, Nullable<double> filterHWE=R_NilValue, Nullable<double> filterMind=R_NilValue, Nullable<double> filterMAF=R_NilValue, int threads=0, bool verbose=true) {
+// [[Rcpp::export]]
+List GenoFilter(const SEXP pBigMat, double NA_C, Nullable<IntegerVector> keepIndsNull=R_NilValue, Nullable<double> filterGeno=R_NilValue, Nullable<double> filterHWE=R_NilValue, Nullable<double> filterMind=R_NilValue, Nullable<double> filterMAF=R_NilValue, int threads=0, bool verbose=true) {
+  omp_setup(threads);
 
-  double m = pMat->nrow(), n = pMat->ncol();
-  IntegerVector  keepRows = seq(0, m - 1);
-  IntegerVector keepCols;
-  if (keepInds.isNull()) {
-    keepCols = seq(0, n - 1);
+  XPtr<BigMatrix> pMat(pBigMat);
+
+  double n = pMat->nrow(), m = pMat->ncol();
+  IntegerVector  keepMrks = seq(0, m - 1);
+  IntegerVector keepInds;
+  if (keepIndsNull.isNull()) {
+    keepInds = seq(0, n - 1);
   } else {
-    keepCols = as<IntegerVector>(keepInds);
-    keepCols = keepCols - 1;
-    n = keepCols.size();
+    keepInds = as<IntegerVector>(keepIndsNull);
+    keepInds = keepInds - 1;
+    n = keepInds.size();
   }
   
   double fgeno = 0, fhwe = 0, fmaf = 0, fmind = 0;
@@ -259,7 +257,7 @@ List GenoFilter(XPtr<BigMatrix> pMat, double NA_C, Nullable<IntegerVector> keepI
   
   if (verbose) {
     Rcout << " Options in effect:" << endl;
-    if (keepInds.isNotNull()  ) { Rcout << "   --keep-ind filePed "       << endl; }
+    if (keepIndsNull.isNotNull()  ) { Rcout << "   --keep-ind filePed "       << endl; }
     if (filterGeno.isNotNull()) { Rcout << "   --geno " << fgeno  << endl; }
     if (filterHWE.isNotNull() ) { Rcout << "   --hwe "  << fhwe   << endl; }
     if (filterMAF.isNotNull() ) { Rcout << "   --maf "  << fmaf   << endl; }
@@ -271,12 +269,12 @@ List GenoFilter(XPtr<BigMatrix> pMat, double NA_C, Nullable<IntegerVector> keepI
   
   if (filterMind.isNotNull()) {
     if (verbose) { Rcout << " Calculating sample missingness rates..."; }
-    NumericVector colNumNA = FilterMind(pMat, NA_C, threads);
+    NumericVector indNumNA = FilterMind(pBigMat, NA_C, threads);
     if (verbose) {  Rcout << " done." << endl; }
-    keepCols = keepCols[colNumNA/m < fmind];
+    keepInds = keepInds[indNumNA/m < fmind];
     if (verbose) {
-      Rcout << " " << (n - keepCols.size())  << " samples removed due to missing genotype data (--mind)." << endl;
-      n = keepCols.size();
+      Rcout << " " << (n - keepInds.size())  << " samples removed due to missing genotype data (--mind)." << endl;
+      n = keepInds.size();
       Rcout << " " << n << " samples remaining after main filters." << endl;
       Rcout << endl;
     }
@@ -284,12 +282,12 @@ List GenoFilter(XPtr<BigMatrix> pMat, double NA_C, Nullable<IntegerVector> keepI
   
   if (filterGeno.isNotNull()) {
     if (verbose) { Rcout << " Calculating variant missingness rates..."; }
-    NumericVector rowNumNA = FilterGeno(pMat, NA_C, keepRows, keepCols, threads);
+    NumericVector mrkNumNA = FilterGeno(pBigMat, NA_C, keepInds, keepMrks, threads);
     if (verbose) {  Rcout << " done." << endl; }
-    keepRows = keepRows[rowNumNA/n < fgeno];
+    keepMrks = keepMrks[mrkNumNA/n < fgeno];
     if (verbose) {
-      Rcout << " " << (m - keepRows.size()) << " variants removed due to missing genotype data (--geno)." << endl;
-      m = keepRows.size();
+      Rcout << " " << (m - keepMrks.size()) << " variants removed due to missing genotype data (--geno)." << endl;
+      m = keepMrks.size();
       Rcout << " " << m << " variants remaining after main filters." << endl;
       Rcout << endl;
     }
@@ -297,8 +295,8 @@ List GenoFilter(XPtr<BigMatrix> pMat, double NA_C, Nullable<IntegerVector> keepI
   
   arma::mat genoFreq;
   if (filterMAF.isNotNull() || filterHWE.isNotNull()) {
-    if (verbose) { Rcout << " Calculating Genotype Frequencies..."; }
-    genoFreq = CalGenoFreq(pMat, keepRows, keepCols, threads);
+    if (verbose) { Rcout << " Calculating Genotype FrequenindIdxes..."; }
+    genoFreq = CalGenoFreq(pBigMat, keepInds, keepMrks, threads);
     if (verbose) {  Rcout << " done." << endl << endl; }
   }
   
@@ -306,230 +304,257 @@ List GenoFilter(XPtr<BigMatrix> pMat, double NA_C, Nullable<IntegerVector> keepI
     if (verbose) { Rcout << " Performing Hardy-Weinberg test..."; }
     NumericVector PVAL = FilterHWE(genoFreq, threads);
     if (verbose) {  Rcout << " done." << endl; }
-    keepRows = keepRows[PVAL > fhwe];
+    keepMrks = keepMrks[PVAL > fhwe];
     arma::vec armaPVAL = as<arma::vec>(PVAL);
     genoFreq = genoFreq.rows(arma::find(armaPVAL > fhwe));
     if (verbose) {
-      Rcout << " " << (m - keepRows.size()) << " variants removed due to exceeding HWE-P-Value (--hwe)." << endl;
-      m = keepRows.size();
+      Rcout << " " << (m - keepMrks.size()) << " variants removed due to exceeding HWE-P-Value (--hwe)." << endl;
+      m = keepMrks.size();
       Rcout << " " << m << " variants remaining after main filters." << endl;
       Rcout << endl;
     }
   }
   
   if (filterMAF.isNotNull()) {
-    if (verbose) { Rcout << " Calculating Minor Allele Frequencies..."; }
+    if (verbose) { Rcout << " Calculating Minor Allele FrequenindIdxes..."; }
     NumericVector MAF = FilterMAF(genoFreq, threads);
     if (verbose) {  Rcout << " done." << endl; }
-    keepRows = keepRows[MAF >= fmaf];
+    keepMrks = keepMrks[MAF >= fmaf];
     if (verbose) {
-      Rcout << " " << (m - keepRows.size()) << " variants removed due to exceeding MAF (--maf)." << endl;
-      m = keepRows.size();
+      Rcout << " " << (m - keepMrks.size()) << " variants removed due to exceeding MAF (--maf)." << endl;
+      m = keepMrks.size();
       Rcout << " " << m << " variants remaining after main filters." << endl;
       Rcout << endl;
     }
   }
   
-  keepRows = keepRows + 1;
-  keepCols = keepCols + 1;
-  List genoInfo = List::create(Named("keepRows") = keepRows,
-                                   _["keepCols"] = keepCols);
+  keepMrks = keepMrks + 1;
+  keepInds = keepInds + 1;
+  List genoInfo = List::create(Named("keepMrks") = keepMrks,
+                                   _["keepInds"] = keepInds);
   return genoInfo;
 }
 
-// [[Rcpp::export]]
-List GenoFilter(const SEXP pBigMat, Nullable<IntegerVector> keepInds=R_NilValue, Nullable<double> filterGeno=R_NilValue, Nullable<double> filterHWE=R_NilValue, Nullable<double> filterMind=R_NilValue, Nullable<double> filterMAF=R_NilValue, int threads=0, bool verbose=true) {
-  XPtr<BigMatrix> xpMat(pBigMat);
-  
-  switch(xpMat->matrix_type()) {
-  case 1:
-    return GenoFilter<char>(xpMat, NA_CHAR, keepInds, filterGeno, filterHWE, filterMind, filterMAF, threads, verbose);
-  case 2:
-    return GenoFilter<short>(xpMat, NA_SHORT, keepInds, filterGeno, filterHWE, filterMind, filterMAF, threads, verbose);
-  case 4:
-    return GenoFilter<int>(xpMat, NA_INTEGER, keepInds, filterGeno, filterHWE, filterMind, filterMAF, threads, verbose);
-  case 8:
-    return GenoFilter<double>(xpMat, NA_REAL, keepInds, filterGeno, filterHWE, filterMind, filterMAF, threads, verbose);
-  default:
-    throw Rcpp::exception("unknown type detected for big.matrix object!");
-  }
-}
-
 template<typename T>
-void Mat2BigMat(XPtr<BigMatrix> pMat, IntegerMatrix mat, Nullable<IntegerVector> colIdx=R_NilValue, int op=1, int threads=0) {
+void Mat2BigMat(XPtr<BigMatrix> pMat, IntegerMatrix &mat, Nullable<IntegerVector> indIdxNull=R_NilValue, int op=1, int threads=0) {
   omp_setup(threads);
   
   MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
   
-  IntegerVector ci;
-  if (colIdx.isNull()) {
-    ci = seq(0, mat.ncol() - 1);
+  IntegerVector indIdx;
+  if (indIdxNull.isNull()) {
+    indIdx = seq(0, mat.nrow() - 1);
   } else {
-    ci = as<IntegerVector>(colIdx);
-    ci = ci - 1;
+    indIdx = as<IntegerVector>(indIdxNull);
+    indIdx = indIdx - 1;
   }
   
-  size_t i, j, m = mat.nrow(), n = ci.length();
+  size_t i, j, n = indIdx.length(), m = mat.ncol(), n0 = pMat->nrow(), m0 = pMat->ncol();
   op = op - 1;
-  if (m != pMat->nrow()) {
+  if (m != m0) {
     Rcpp::stop("'bigmat' and 'mat' should have the same marker number!");
   }
-  if (op + n > pMat->ncol()) {
+  if (op + n > n0) {
     Rcpp::stop("'mat' cannot be intert to bigmat completely!");
   }
-  if (max(ci) + 1 > mat.ncol()) {
-    Rcpp::stop("'colIdx' is out of bound!");
+  if (max(indIdx) + 1 > mat.nrow()) {
+    Rcpp::stop("'indIdx' is out of bound!");
   }
   
   #pragma omp parallel for schedule(dynamic) private(i, j)
-  for (j = 0; j < n; j++) {
-    for (i = 0; i < m; i++) {
-      bigmat[op + j][i] = mat(i, ci[j]);
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      bigmat[j][op + i] = mat(indIdx[i], j);
     }
   }
 
 }
 
 // [[Rcpp::export]]
-void Mat2BigMat(const SEXP pBigMat, IntegerMatrix mat, Nullable<IntegerVector> colIdx=R_NilValue, int op=1, int threads=0) {
+void Mat2BigMat(const SEXP pBigMat, IntegerMatrix &mat, Nullable<IntegerVector> indIdxNull=R_NilValue, int op=1, int threads=0) {
   XPtr<BigMatrix> xpMat(pBigMat);
   
   switch(xpMat->matrix_type()) {
   case 1:
-    return Mat2BigMat<char>(xpMat, mat, colIdx, op, threads);
+    return Mat2BigMat<char>(xpMat, mat, indIdxNull, op, threads);
   case 2:
-    return Mat2BigMat<short>(xpMat, mat, colIdx, op, threads);
+    return Mat2BigMat<short>(xpMat, mat, indIdxNull, op, threads);
   case 4:
-    return Mat2BigMat<int>(xpMat, mat, colIdx, op, threads);
+    return Mat2BigMat<int>(xpMat, mat, indIdxNull, op, threads);
   case 8:
-    return Mat2BigMat<double>(xpMat, mat, colIdx, op, threads);
+    return Mat2BigMat<double>(xpMat, mat, indIdxNull, op, threads);
   default:
     throw Rcpp::exception("unknown type detected for big.matrix object!");
   }
 }
 
 template<typename T>
-void BigMat2BigMat(XPtr<BigMatrix> pMat, XPtr<BigMatrix> pmat, Nullable<IntegerVector> colIdx=R_NilValue, int op=1, int threads=0) {
+void BigMat2BigMat(XPtr<BigMatrix> pMat, XPtr<BigMatrix> pmat, Nullable<IntegerVector> indIdxNull=R_NilValue, int op=1, int threads=0) {
   omp_setup(threads);
   
   MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
   MatrixAccessor<T> bigm = MatrixAccessor<T>(*pmat);
   
-  IntegerVector ci;
-  if (colIdx.isNull()) {
-    ci = seq(0, pmat->ncol() - 1);
+  IntegerVector indIdx;
+  size_t n1 = pmat->nrow();
+  if (indIdxNull.isNull()) {
+    indIdx = seq(0, n1 - 1);
   } else {
-    ci = as<IntegerVector>(colIdx);
-    ci = ci - 1;
+    indIdx = as<IntegerVector>(indIdxNull);
+    indIdx = indIdx - 1;
   }
   
-  size_t i, j, m = pmat->nrow(), n = ci.length();
+  size_t i, j, n = indIdx.length(), m = pmat->ncol(), n0 = pMat->nrow(), m0 = pMat->ncol();
   op = op - 1;
-  if (m != pMat->nrow()) {
+  if (m != m0) {
     Rcpp::stop("'bigmat' and 'pmat' should have the same marker number!");
   }
-  if (op + n > pMat->ncol()) {
+  if (op + n > n0) {
     Rcpp::stop("'pmat' cannot be intert to bigmat completely!");
   }
-  if (max(ci) + 1 > pmat->ncol()) {
-    Rcpp::stop("'colIdx' is out of bound!");
+  size_t maxIndIdx = max(indIdx);
+  if (maxIndIdx + 1 > n1) {
+    Rcpp::stop("'indIdx' is out of bound!");
   }
   
-  IntegerMatrix mat(pmat->nrow(), pmat->ncol());
+  IntegerMatrix mat(n1, m);
   #pragma omp parallel for schedule(dynamic) private(i, j)
-  for (j = 0; j < pmat->ncol(); j++) {
-    for (i = 0; i < m; i++) {
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n1; i++) {
       mat(i, j) = bigm[j][i];
     }
   }
   
   #pragma omp parallel for schedule(dynamic) private(i, j)
-  for (j = 0; j < n; j++) {
-    for (i = 0; i < m; i++) {
-      bigmat[op + j][i] = mat(i, ci[j]);
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      bigmat[j][op + i] = mat(indIdx[i], j);
     }
   }
   
 }
 
 // [[Rcpp::export]]
-void BigMat2BigMat(const SEXP pBigMat, const SEXP pBigmat, Nullable<IntegerVector> colIdx=R_NilValue, int op=1, int threads=0) {
+void BigMat2BigMat(const SEXP pBigMat, const SEXP pBigmat, Nullable<IntegerVector> indIdxNull=R_NilValue, int op=1, int threads=0) {
   XPtr<BigMatrix> xpMat(pBigMat);
   XPtr<BigMatrix> xpmat(pBigmat);
   
   switch(xpMat->matrix_type()) {
   case 1:
-    return BigMat2BigMat<char>(xpMat, xpmat, colIdx, op, threads);
+    return BigMat2BigMat<char>(xpMat, xpmat, indIdxNull, op, threads);
   case 2:
-    return BigMat2BigMat<short>(xpMat, xpmat, colIdx, op, threads);
+    return BigMat2BigMat<short>(xpMat, xpmat, indIdxNull, op, threads);
   case 4:
-    return BigMat2BigMat<int>(xpMat, xpmat, colIdx, op, threads);
+    return BigMat2BigMat<int>(xpMat, xpmat, indIdxNull, op, threads);
   case 8:
-    return BigMat2BigMat<double>(xpMat, xpmat, colIdx, op, threads);
+    return BigMat2BigMat<double>(xpMat, xpmat, indIdxNull, op, threads);
   default:
     throw Rcpp::exception("unknown type detected for big.matrix object!");
   }
 }
 
 template<typename T>
-void GenoMixer(XPtr<BigMatrix> pMat, XPtr<BigMatrix> pmat, IntegerVector sirIdx, IntegerVector damIdx, int nBlock=100, int op=1, int threads=0) {
+void geno_cvt1_mat(XPtr<BigMatrix> pMat, IntegerMatrix &mat, int threads=0) {
+  omp_setup(threads);
+  
+  MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
+
+  size_t i, j, n = pMat->nrow(), m = pMat->ncol();
+
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      bigmat[j][i] = mat(2*i, j) + mat(2*i+1, j);
+    }
+  }
+  
+}
+
+// [[Rcpp::export]]
+void geno_cvt1_mat(const SEXP pBigMat, IntegerMatrix &mat, int threads=0) {
+  XPtr<BigMatrix> xpMat(pBigMat);
+  
+  switch(xpMat->matrix_type()) {
+  case 1:
+    return geno_cvt1_mat<char>(xpMat, mat, threads);
+  case 2:
+    return geno_cvt1_mat<short>(xpMat, mat, threads);
+  case 4:
+    return geno_cvt1_mat<int>(xpMat, mat, threads);
+  case 8:
+    return geno_cvt1_mat<double>(xpMat, mat, threads);
+  default:
+    throw Rcpp::exception("unknown type detected for big.matrix object!");
+  }
+}
+
+template<typename T>
+void geno_cvt1_bigmat(XPtr<BigMatrix> pMat, XPtr<BigMatrix> pmat, int threads=0) {
   omp_setup(threads);
   
   MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
   MatrixAccessor<T> bigm = MatrixAccessor<T>(*pmat);
-  
-  sirIdx = sirIdx - 1;
-  damIdx = damIdx - 1;
-  
-  size_t op_row, ed_row, i, j, k, m, n, judpar, kidIdx;
-  std::random_device rd;
-  m = pmat->nrow(); 
-  n = damIdx.length();
-  op = op - 1;
-  
-  if (m != pMat->nrow()) {
-    Rcpp::stop("'bigmat' and 'pmat' should have the same marker number!");
-  }
-  if (op + n > pMat->ncol()) {
-    Rcpp::stop("'pmat' cannot be intert to bigmat completely!");
-  }
-  if ((max(sirIdx) > pmat->ncol()) || (max(damIdx) > pmat->ncol())) {
-    Rcpp::stop("'sirIdx' or 'damIdx' is out of bound!");
-  }
-  if (sirIdx.length() != damIdx.length()) {
-    Rcpp::stop("'sirIdx' and 'damIdx' should have the same length!");
-  }
-  
-  int len_block = m / nBlock;
-  int tail_block = m % nBlock + len_block;
-  IntegerVector nInblock(nBlock);
-  IntegerVector accum_block(nBlock);
-  for (i = 0; i < nBlock; i++) {
-    nInblock[i] = len_block;
-  }
-  nInblock[nBlock - 1] = tail_block;
-  accum_block[0] = nInblock[0];
-  for (i = 1; i < nBlock; i++) {
-    accum_block[i] = accum_block[i - 1] + nInblock[i];
-  }
-  
-  IntegerMatrix mat(pmat->nrow(), pmat->ncol());
+
+  size_t i, j, n = pMat->nrow(), m = pMat->ncol();
+
+  IntegerMatrix mat(n, m);
   #pragma omp parallel for schedule(dynamic) private(i, j)
-  for (j = 0; j < pmat->ncol(); j++) {
-    for (i = 0; i < m; i++) {
-      mat(i, j) = bigm[j][i];
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      mat(i, j) = bigm[j][2*i] + bigm[j][2*i+1];
+    }
+  }
+
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      bigmat[j][i] = mat(i, j);
     }
   }
   
-  #pragma omp parallel for schedule(dynamic) private(i, j, k, op_row, ed_row, judpar, kidIdx)
-  for (k = 0; k < nBlock; k++) {
-    ed_row = accum_block[k];
-    op_row = ed_row - nInblock[k];
-    for (j = 0; j < n; j++) {
-      judpar = rd();
-      kidIdx = judpar % 2 == 0 ? sirIdx[j] : damIdx[j];
-      for (i = op_row; i < ed_row; i++) {
-        bigmat[op + j][i] = mat(i, kidIdx);
+}
+
+// [[Rcpp::export]]
+void geno_cvt1_bigmat(const SEXP pBigMat, const SEXP pBigmat, int threads=0) {
+  XPtr<BigMatrix> xpMat(pBigMat);
+  XPtr<BigMatrix> xpmat(pBigmat);
+  
+  switch(xpMat->matrix_type()) {
+  case 1:
+    return geno_cvt1_bigmat<char>(xpMat, xpmat, threads);
+  case 2:
+    return geno_cvt1_bigmat<short>(xpMat, xpmat, threads);
+  case 4:
+    return geno_cvt1_bigmat<int>(xpMat, xpmat, threads);
+  case 8:
+    return geno_cvt1_bigmat<double>(xpMat, xpmat, threads);
+  default:
+    throw Rcpp::exception("unknown type detected for big.matrix object!");
+  }
+}
+
+template<typename T>
+void geno_cvt2_mat(XPtr<BigMatrix> pMat, IntegerMatrix &mat, int threads=0) {
+  omp_setup(threads);
+  
+  MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
+
+  size_t i, j, n = mat.nrow(), m = mat.ncol();
+
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      if (mat(i, j) == 0) {
+        bigmat[j][2*i] = 0;
+        bigmat[j][2*i+1] = 0;
+      } else if (mat(i, j) == 1) {
+        bigmat[j][2*i] = 0;
+        bigmat[j][2*i+1] = 1;
+      } else if (mat(i, j) == 2) {
+        bigmat[j][2*i] = 1;
+        bigmat[j][2*i+1] = 1;
+      } else {
+        Rcpp::stop("Elements in genotype data should be 0, 1 or 2!");
       }
     }
   }
@@ -537,19 +562,154 @@ void GenoMixer(XPtr<BigMatrix> pMat, XPtr<BigMatrix> pmat, IntegerVector sirIdx,
 }
 
 // [[Rcpp::export]]
-void GenoMixer(const SEXP pBigMat, const SEXP pBigmat, IntegerVector sirIdx, IntegerVector damIdx, int nBlock=100, int op=1, int threads=0) {
+void geno_cvt2_mat(const SEXP pBigMat, IntegerMatrix &mat, int threads=0) {
+  XPtr<BigMatrix> xpMat(pBigMat);
+  
+  switch(xpMat->matrix_type()) {
+  case 1:
+    return geno_cvt2_mat<char>(xpMat, mat, threads);
+  case 2:
+    return geno_cvt2_mat<short>(xpMat, mat, threads);
+  case 4:
+    return geno_cvt2_mat<int>(xpMat, mat, threads);
+  case 8:
+    return geno_cvt2_mat<double>(xpMat, mat, threads);
+  default:
+    throw Rcpp::exception("unknown type detected for big.matrix object!");
+  }
+}
+
+template<typename T>
+void geno_cvt2_bigmat(XPtr<BigMatrix> pMat, XPtr<BigMatrix> pmat, int threads=0) {
+  omp_setup(threads);
+  
+  MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
+  MatrixAccessor<T> bigm = MatrixAccessor<T>(*pmat);
+
+  size_t i, j, n = pmat->nrow(), m = pmat->ncol();
+
+  IntegerMatrix mat(2 * n, m);
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+    for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      if (bigm[j][i] == 0) {
+        mat(2*i, j) = 0;
+        mat(2*i+1, j) = 0;
+      } else if (bigm[j][i] == 1) {
+        mat(2*i, j) = 0;
+        mat(2*i+1, j) = 1;
+      } else if (bigm[j][i] == 2) {
+        mat(2*i, j) = 1;
+        mat(2*i+1, j) = 1;
+      } else {
+        Rcpp::stop("Elements in genotype data should be 0, 1 or 2!");
+      }
+    }
+  }
+
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < 2 * n; i++) {
+      bigmat[j][i] = mat(i, j);
+    }
+  }
+  
+}
+
+// [[Rcpp::export]]
+void geno_cvt2_bigmat(const SEXP pBigMat, const SEXP pBigmat, int threads=0) {
   XPtr<BigMatrix> xpMat(pBigMat);
   XPtr<BigMatrix> xpmat(pBigmat);
   
   switch(xpMat->matrix_type()) {
   case 1:
-    return GenoMixer<char>(xpMat, xpmat, sirIdx, damIdx, nBlock, op, threads);
+    return geno_cvt2_bigmat<char>(xpMat, xpmat, threads);
   case 2:
-    return GenoMixer<short>(xpMat, xpmat, sirIdx, damIdx, nBlock, op, threads);
+    return geno_cvt2_bigmat<short>(xpMat, xpmat, threads);
   case 4:
-    return GenoMixer<int>(xpMat, xpmat, sirIdx, damIdx, nBlock, op, threads);
+    return geno_cvt2_bigmat<int>(xpMat, xpmat, threads);
   case 8:
-    return GenoMixer<double>(xpMat, xpmat, sirIdx, damIdx, nBlock, op, threads);
+    return geno_cvt2_bigmat<double>(xpMat, xpmat, threads);
+  default:
+    throw Rcpp::exception("unknown type detected for big.matrix object!");
+  }
+}
+
+template<typename T>
+void bigt_mat(XPtr<BigMatrix> pMat, IntegerMatrix &mat, int threads=0) {
+  omp_setup(threads);
+  
+  MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
+
+  size_t i, j, n = pMat->nrow(), m = pMat->ncol();
+
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      bigmat[j][i] = mat(j, i);
+    }
+  }
+  
+}
+
+// [[Rcpp::export]]
+void bigt_mat(const SEXP pBigMat, IntegerMatrix &mat, int threads=0) {
+  XPtr<BigMatrix> xpMat(pBigMat);
+  
+  switch(xpMat->matrix_type()) {
+  case 1:
+    return bigt_mat<char>(xpMat, mat, threads);
+  case 2:
+    return bigt_mat<short>(xpMat, mat, threads);
+  case 4:
+    return bigt_mat<int>(xpMat, mat, threads);
+  case 8:
+    return bigt_mat<double>(xpMat, mat, threads);
+  default:
+    throw Rcpp::exception("unknown type detected for big.matrix object!");
+  }
+}
+
+template<typename T>
+void bigt_bigmat(XPtr<BigMatrix> pMat, XPtr<BigMatrix> pmat, int threads=0) {
+  omp_setup(threads);
+  
+  MatrixAccessor<T> bigmat = MatrixAccessor<T>(*pMat);
+  MatrixAccessor<T> bigm = MatrixAccessor<T>(*pmat);
+
+  size_t i, j, n = pMat->nrow(), m = pMat->ncol();
+
+  IntegerMatrix mat(2 * n, m);
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      mat(i, j) = bigm[i][j];
+    }
+  }
+
+  #pragma omp parallel for schedule(dynamic) private(i, j)
+  for (j = 0; j < m; j++) {
+    for (i = 0; i < n; i++) {
+      bigmat[j][i] = mat(i, j);
+    }
+  }
+  
+}
+
+// [[Rcpp::export]]
+void bigt_bigmat(const SEXP pBigMat, const SEXP pBigmat, int threads=0) {
+  XPtr<BigMatrix> xpMat(pBigMat);
+  XPtr<BigMatrix> xpmat(pBigmat);
+  
+  switch(xpMat->matrix_type()) {
+  case 1:
+    return bigt_bigmat<char>(xpMat, xpmat, threads);
+  case 2:
+    return bigt_bigmat<short>(xpMat, xpmat, threads);
+  case 4:
+    return bigt_bigmat<int>(xpMat, xpmat, threads);
+  case 8:
+    return bigt_bigmat<double>(xpMat, xpmat, threads);
   default:
     throw Rcpp::exception("unknown type detected for big.matrix object!");
   }
